/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 * 
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-share/src/org/opendj/share/Util.java,v 1.1.1.1 2001/09/29 20:39:30 jonathan Exp $
 * $Log: Util.java,v $
 * Revision 1.1.1.1  2001/09/29 20:39:30  jonathan
 * Imported opendj-share
 *
 */
package org.opendj.share;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.BufferedReader;
import java.io.StringWriter;
import java.io.PrintWriter;
import java.util.Properties;
import java.util.Enumeration;

public class Util {

    public static final long MINS_IN_DAY = 60 * 24;
    public static final long MILLIS_IN_DAY = 1000 * 60 * MINS_IN_DAY;

    public static final long MILLIS_IN_MINUTE = 1000 * 60;
    public static final long MILLIS_IN_HOUR   = MILLIS_IN_MINUTE * 60;

    public static final String EMPTY_STRING = "";

    /**
     * Gets a stack trace from the Throwable as a String.
     * If the Throwable is an instance of NestedException, the
     * full, nested stack trace is returned.
     *
     * @param theT the Throwable object to get a stack trace for
     * @return the full stack trace
     */
    public static String getStackTrace( Throwable theT ) {
        if ( theT == null ) return "THROWABLE-WAS-NULL";
        try {
            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter(sw);
            theT.printStackTrace(pw);
            String rval = sw.toString();
            pw.close();
            if ( theT instanceof NestedException && 
                 ((NestedException) theT).getBaseThrowable() != null ) {
                rval += "\nNESTED STACK TRACE:\n"
                     + getStackTrace( ((NestedException)theT).getBaseThrowable() );
            }
            return rval;
        } catch ( Exception e ) {
            return "\n\nUtil.getStackTrace GENERATED EXCEPTION: '" 
                + e.toString() + "'\n\n";
        }
    }

    /**
     * @param theProps the Properties object to format
     * @return a formatted String representing all properties in theProps
     */
    public static String getPropertiesAsString( Properties theProps ) {
        if ( theProps == null ) return null;
        Enumeration somePropNames = theProps.propertyNames();
        String rstr = Util.EMPTY_STRING;
        String aPropName = null;
        while ( somePropNames.hasMoreElements() ) {
            aPropName = (String) somePropNames.nextElement();
            rstr += aPropName + "==>" + theProps.getProperty(aPropName) + "\n";
        }
        return rstr;
    }

    /**
     * Sleeps for the specified period of time, unless interrupted.
     * The interrupt is ignored.
     *
     * @param ms the number of milliseconds to sleep
     */
    public static void sleep ( long ms ) {
        // debug.out("Util.sleep(" + ms + ") from: " + Util.getStackTrace(new Exception()));
        try {
            Thread.sleep(ms);
        } catch ( InterruptedException e ) {}
    }

    /**
     * Replaces all occurences of find with replace
     *
     * @param src the source string
     * @param find the string to find
     * @param replace the string to use to replace <code>find</code>
     * @return the source string with all occurrences of <code>find</code> replaced by <code>replace</code>
     */
    public static String stringReplace ( String src, String find, String replace ) {

        if ( src              == null || 
             find             == null ||
             replace          == null || 
             src.length()     == 0    ||
             find.length()    == 0    ||
             replace.length() == 0 ) {

            return src;
        }

        String rstr = "";
        int idx = src.indexOf(find);
        int startIdx = 0;
        while ( idx != -1 ) {
            rstr += src.substring(startIdx, idx) + replace;
            startIdx = idx + find.length();
            idx = src.indexOf(find, startIdx);
        }
        rstr += src.substring(startIdx, src.length());

        return rstr;        
    }

    /**
     * Capitalize a word.
     *
     * @param s the String to capitalize
     * @return s with the first letter in upper case, or null if s was null
     */
    public static String capitalize( String s ) {
        return ( s == null || s.length() == 0 ) ? s : ( s.length() == 1 ) ? s.toUpperCase() : s.substring(0,1).toUpperCase() + s.substring(1);
    }

    /**
     * Formats a millisecond value into "X days, Y hours, Z mins, W secs"
     */
    public static String formatMillis ( long theMillis ) {
        
        String rstr = "";
        long aWorkingMillis = theMillis;
        
        long aNumDays = aWorkingMillis / Util.MILLIS_IN_DAY;
        aWorkingMillis -= (aNumDays*Util.MILLIS_IN_DAY);
        
        long aNumHours = aWorkingMillis / Util.MILLIS_IN_HOUR;
        aWorkingMillis -= (aNumHours*Util.MILLIS_IN_HOUR);
        
        long aNumMins =  aWorkingMillis / Util.MILLIS_IN_MINUTE;
        aWorkingMillis -= (aNumMins*Util.MILLIS_IN_MINUTE);
        
        long aNumSecs =  aWorkingMillis / 1000;
        aWorkingMillis -= (aNumSecs*1000);
        
        long aNumMillis = aWorkingMillis;

        boolean needComma = false;
        if ( aNumDays > 0 ) {
            rstr += String.valueOf(aNumDays) + " days";
            needComma = true;
        }
        if ( aNumHours > 0 ) {
            if ( needComma ) rstr += ",";
            rstr += String.valueOf(aNumHours) + " hours";
            needComma = true;
        }
        if ( aNumMins > 0 ) {
            if ( needComma ) rstr += ",";
            rstr += String.valueOf(aNumMins) + " mins";
            needComma = true;
        }
        if ( aNumSecs > 0 ) {
            if ( needComma ) rstr += ",";
            rstr += String.valueOf(aNumSecs) + " secs";
            needComma = true;
        }

        return rstr;
    }

    /**
     * Read an entire stream and return it as a String
     *
     * @param theInStream the InputStream to read.
     * @return a String representing all text read.
     * @exception IOException if an IO error occurs.
     */
    public static String readStreamAsString ( InputStream theInStream ) throws IOException {
        BufferedReader aReader = new BufferedReader(new InputStreamReader(theInStream));
        String aLine = aReader.readLine();
        String rstr = "";
        while ( aLine != null ) {
            rstr += aLine + "\n";
            aLine = aReader.readLine();
        }
        return rstr;
    }

    /**
     * Change a null String to a zero-length String
     * @param s the String
     * @return s if s is not null, otherwise, return the empty string
     */
    public static String nz( String s ) {
        return ( s == null ) ? EMPTY_STRING : s;
    }

    /**
     * This method removes the last occurrence of theDelim from theString, and
     * any text that occurs before theDelim as well
     *
     * @param theString the source String
     * @param theDelim the delimiter text
     * @return a modified String, with theDelim and any preceding text removed.
     */
    public static String stripUpTo ( String theString, String theDelim ) {
        if ( theString == null || theDelim == null ) return theString;
        int aDelimPos = theString.lastIndexOf(theDelim);
        if ( aDelimPos == -1 ) return theString;
        return theString.substring(aDelimPos + theDelim.length());
    }

    /**
     * Copy all data from an InputStream to an OutputStream
     *
     * @param theInStream the InputStream to read
     * @param theOutStream the OutputStream to write
     * @exception IOException if any IO error occurs.
     */
    public static void copyStream( InputStream theInStream, OutputStream theOutStream ) throws IOException {
        
        int aNumBytesRead = 0;
        byte[] aBuffer = new byte[DEFAULT_BUFFER_SIZE];
        while ( aNumBytesRead >= 0 ) { 
            aNumBytesRead = theInStream.read(aBuffer, 0, aBuffer.length);
            if ( aNumBytesRead > 0 ) {
                theOutStream.write(aBuffer, 0, aNumBytesRead);
            }
        }
    }
    public static final int DEFAULT_BUFFER_SIZE = 4096;

    /**
     * Given a string to match and an array of strings, returns the index in the
     * array where the string was found, or -1 if not found.  Note that the comparison
     * is CASE SENSITIVE
     *
     * @param theString the String to search for in the array
     * @param theStringArray the array of Strings to search
     * @return the index of theString in the theStringArray, or -1 if it could not be found.
     */
    public static int getStringIndex ( String theString, String[] theStringArray ) {
        for ( int i=0; i<theStringArray.length; i++ ) {
            if ( theStringArray[i] != null && theStringArray[i].equals(theString) ) {
                return i;
            }
        }
        return -1;
    }

}
